clear
clc

openfemm; % opens FEMM

% Constants (kms)
mag_dep = 0.0254*1.875; % magnet depth [m]
mag_width = 0.0254*0.875; % magnet width [m]
mag_height = 0.0254*0.375; % magnet height [m]
gauge = 10; % armature conductor gauge [AWG]
ds = (0.127e-3)*92^((36-gauge)/39); % conversion of AWG to diameter [m]
spppp = 2;% slots per pole per phase
poles = 16; % number of poles
phase = 3; % number of phases
mag_r = 0.0254*5.2/2; % radius to center of magnet [m]
wire_r = 0.0254*4.5/2; % radius to center of conductor [m]
I = 0; % coil current [A]
pie = 2*pi/(poles/2); % angle spanned by one pole pair [rad]

newdocument(0); % create magnetic problem
mi_getmaterial('Copper'); % add copper material from library to list
mi_getmaterial('Air'); % add air material from library to list
mi_getmaterial('Ceramic 8'); % % add Ceramic grade 8 material from library to list
mi_addcircprop('Coil',I,0); % create one coil
mi_addblocklabel(1e-3*cos(pie/2),1e-3*sin(pie/2)); % add label for air
mi_selectlabel(1e-3,1e-3); % select label
mi_setblockprop('Air',0,0,'<None>',0,0,0); % set to air

% conductors
dtheta = 2*pi/(2*phase*spppp)/(poles/2); % angle between conductors
for n = 0:(2*phase*spppp-1) % iterate through conductors in pole pair
    x = wire_r*cos(n*dtheta+dtheta/2); % x of center of conductor
    y = wire_r*sin(n*dtheta+dtheta/2); % y of center of conductor
    mi_drawarc(x-ds/2,y,x+ds/2,y,180,1); % draw bottom half of circle
    mi_addarc(x+ds/2,y,x-ds/2,y,180,1); % draw top half of circle
    mi_addblocklabel(x,y); % add label to center of conductor
    mi_selectlabel(x,y); % select label
    mi_setblockprop('Copper',0,0,'Coil',0,0,1); % set to copper and coil
    mi_setgroup(1); % set conductor to coil group
    mi_clearselected;
end

% magnets
mi_drawline(mag_r-mag_height/2,mag_width/2,mag_r-mag_height/2,-mag_width/2);
mi_selectsegment(mag_r-mag_height/2,0);
mi_copyrotate2(0,0,3*180*pie/4/pi,1,1);
mi_selectsegment(mag_r-mag_height/2,0);
mi_deleteselected;
mi_clearselected;
mi_selectsegment((mag_r-mag_height/2)*cos(3*pie/4),(mag_r-mag_height/2)*sin(3*pie/4));
mi_copyrotate2(0,0,-180*pie/2/pi,1,1);
mi_clearselected;
mi_selectnode(mag_r-mag_height/2,mag_width/2);
mi_selectnode(mag_r-mag_height/2,-mag_width/2);
mi_deleteselected;
mi_clearselected;

mi_drawline(mag_r+mag_height/2,mag_width/2,mag_r+mag_height/2,-mag_width/2);
mi_selectsegment(mag_r+mag_height/2,0);
mi_copyrotate2(0,0,3*180*pie/4/pi,1,1);
mi_selectsegment(mag_r+mag_height/2,0);
mi_deleteselected;
mi_clearselected;
mi_selectsegment((mag_r+mag_height/2)*cos(3*pie/4),(mag_r+mag_height/2)*sin(3*pie/4));
mi_copyrotate2(0,0,-180*pie/2/pi,1,1);
mi_clearselected;
mi_selectnode(mag_r+mag_height/2,mag_width/2);
mi_selectnode(mag_r+mag_height/2,-mag_width/2);
mi_deleteselected;
mi_clearselected;

mi_drawline(0,0,mag_r+.0254*2,0);
mi_drawline(0,0,(mag_r+.0254*2)*cos(pie),(mag_r+.0254*2)*sin(pie));
mi_drawarc(mag_r+.0254*2,0,(mag_r+.0254*2)*cos(pie),(mag_r+.0254*2)*sin(pie),pie*180/pi,pie*180/pi);

%%
N = 5; % number of sorounding strands
I = 3e-3; % current injection, peak
rho_cu = 1.724e-8;
xp = linspace(3,6,20);
f = 10.^(xp);

Fr100 = []; % Rac/Rdc matrix for ds/100 insulation thickness
Fr10 = []; % Rac/Rdc matrix for ds/10 insulation thickness
Fr1 = []; % Rac/Rdc matrix for ds/1 insulation thickness

for di = 1:((length(AWG)+1)/2) % 16-48 AWG in increments of 2
    for insi = 1:4
        if(insi == 1)
            newdocument(0);
            mi_getmaterial('Copper');
            mi_getmaterial('Air');
            mi_addcircprop('Coil',I,0);
            mi_addblocklabel(0,0);
            mi_selectlabel(0,0);
            mi_setblockprop('Air',0,1,'<None>',0,0,0);
            mi_clearselected;
            ds = AWG(2*di-1); % strand diameter
            d0 = ds*(1+1/50); % effective diameter with insualtion
            r = (d0/2)/sin(pi/N); % helical radius for touching circles
            mi_makeABC(7,3*r,0,0,0);
            dtheta = 2*pi/N;
            for m=0:(N-1)
                x = r*cos(m*dtheta);
                y = r*sin(m*dtheta);
                mi_drawarc(x-ds/2,y,x+ds/2,y,180,1);
                mi_addarc(x+ds/2,y,x-ds/2,y,180,1);
                mi_addblocklabel(x,y);
                mi_selectlabel(x,y);
                mi_setblockprop('Copper',0,1,'Coil',0,0,1);
                mi_setgroup(1);
                mi_clearselected;
            end
            mi_saveas('../FEMM Models/five_bundle.fem');
            for fi = 1:length(f)
                opendocument('../FEMM Models/five_bundle.fem');
                mi_probdef(f(fi),'meters','planar',1e-8,L,30,0);
                mi_saveas('../FEMM Models/five_bundle.fem');
                mi_analyze;
                mi_loadsolution;
                mo_groupselectblock(1);
                Rac = 2*real(mo_blockintegral(4))/(L*I^2); % get resistive losses in slab
                Rdc = rho_cu/(N*pi*(ds/2)^2);
                Fr100(di,fi) = Rac/Rdc;
                mo_clearblock;
            end
            mi_close;
        elseif(insi == 2)
            newdocument(0);
            mi_getmaterial('Copper');
            mi_getmaterial('Air');
            mi_addcircprop('Coil',I,0);
            mi_addblocklabel(0,0);
            mi_selectlabel(0,0);
            mi_setblockprop('Air',0,1,'<None>',0,0,0);
            mi_clearselected;
            ds = AWG(2*di-1); % strand diameter
            d0 = ds*(1+1/5); % effective diameter with insualtion
            r = (d0/2)/sin(pi/N); % helical radius for touching circles
            mi_makeABC(7,3*r,0,0,0);
            dtheta = 2*pi/N;
            for m=0:(N-1)
                x = r*cos(m*dtheta);
                y = r*sin(m*dtheta);
                mi_drawarc(x-ds/2,y,x+ds/2,y,180,1);
                mi_addarc(x+ds/2,y,x-ds/2,y,180,1);
                mi_addblocklabel(x,y);
                mi_selectlabel(x,y);
                mi_setblockprop('Copper',0,1,'Coil',0,0,1);
                mi_setgroup(1);
                mi_clearselected;
            end
            mi_saveas('../FEMM Models/five_bundle.fem');
            for fi = 1:length(f)
                opendocument('../FEMM Models/five_bundle.fem');
                mi_probdef(f(fi),'meters','planar',1e-8,L,30,0);
                mi_saveas('../FEMM Models/five_bundle.fem');
                mi_analyze;
                mi_loadsolution;
                mo_groupselectblock(1);
                Rac = 2*real(mo_blockintegral(4))/(L*I^2); % get resistive losses in slab
                Rdc = rho_cu/(N*pi*(ds/2)^2);
                Fr10(di,fi) = Rac/Rdc;
                mo_clearblock;
            end
            mi_close;
        elseif(insi == 3)
            newdocument(0);
            mi_getmaterial('Copper');
            mi_getmaterial('Air');
            mi_addcircprop('Coil',I,0);
            mi_addblocklabel(0,0);
            mi_selectlabel(0,0);
            mi_setblockprop('Air',0,1,'<None>',0,0,0);
            mi_clearselected;
            ds = AWG(2*di-1); % strand diameter
            d0 = ds*(1+2); % effective diameter with insualtion
            r = (d0/2)/sin(pi/N); % helical radius for touching circles
            mi_makeABC(7,3*r,0,0,0);
            dtheta = 2*pi/N;
            for m=0:(N-1)
                x = r*cos(m*dtheta);
                y = r*sin(m*dtheta);
                mi_drawarc(x-ds/2,y,x+ds/2,y,180,1);
                mi_addarc(x+ds/2,y,x-ds/2,y,180,1);
                mi_addblocklabel(x,y);
                mi_selectlabel(x,y);
                mi_setblockprop('Copper',0,1,'Coil',0,0,1);
                mi_setgroup(1);
                mi_clearselected;
            end
            mi_saveas('../FEMM Models/five_bundle.fem');
            for fi = 1:length(f)
                opendocument('../FEMM Models/five_bundle.fem');
                mi_probdef(f(fi),'meters','planar',1e-8,L,30,0);
                mi_saveas('../FEMM Models/five_bundle.fem');
                mi_analyze;
                mi_loadsolution;
                mo_groupselectblock(1);
                Rac = 2*real(mo_blockintegral(4))/(L*I^2); % get resistive losses in slab
                Rdc = rho_cu/(N*pi*(ds/2)^2);
                Fr1(di,fi) = Rac/Rdc;
                mo_clearblock;
            end
            mi_close;
        else
            newdocument(0);
            mi_getmaterial('Copper');
            mi_getmaterial('Air');
            mi_addcircprop('Coil',I,0);
            mi_addblocklabel(0,0);
            mi_selectlabel(0,0);
            mi_setblockprop('Air',0,1,'<None>',0,0,0);
            mi_clearselected;
            ds = AWG(2*di-1); % strand diameter
            d0 = ds*(1+1); % effective diameter with insualtion
            r = (d0/2)/sin(pi/N); % helical radius for touching circles
            mi_makeABC(7,3*r,0,0,0);
            dtheta = 2*pi/N;
            for m=0:(N-1)
                x = r*cos(m*dtheta);
                y = r*sin(m*dtheta);
                mi_drawarc(x-ds/2,y,x+ds/2,y,180,1);
                mi_addarc(x+ds/2,y,x-ds/2,y,180,1);
                mi_addblocklabel(x,y);
                mi_selectlabel(x,y);
                mi_setblockprop('Copper',0,1,'Coil',0,0,1);
                mi_setgroup(1);
                mi_clearselected;
            end
            mi_saveas('../FEMM Models/five_bundle.fem');
            for fi = 1:length(f)
                opendocument('../FEMM Models/five_bundle.fem');
                mi_probdef(f(fi),'meters','planar',1e-8,L,30,0);
                mi_saveas('../FEMM Models/five_bundle.fem');
                mi_analyze;
                mi_loadsolution;
                mo_groupselectblock(1);
                Rac = 2*real(mo_blockintegral(4))/(L*I^2); % get resistive losses in slab
                Rdc = rho_cu/(N*pi*(ds/2)^2);
                Fr2(di,fi) = Rac/Rdc;
                mo_clearblock;
            end
            mi_close;
        end
    end
end